local gr = love.graphics

function love.load()
	screen = { w = 800, h = 600 }
	love.window.setTitle( 'Flood Fill Test' )
	love.window.setMode( screen.w, screen.h, {borderless=true} )
	gr.setBackgroundColor( 0, 0, 0 )
	canvas = gr.newCanvas( screen.w, screen.h )
	love.mouse.setVisible( false )
	borderColor	= { 1, 1, 0 }
	colors = {
		{1,1,0 },
		{0,1,0 },
		{0,0,1 },
		{1,0,1 },
		{1,1,1 },
		{0,1,1 },
		{1,0,0 },
		{1,1,1 }}	-- Note: fill color ~= borderColor or black!
	fillColor = 1
	gr.setLineWidth( 2 )	-- to prevent leakage
	gr.setCanvas( canvas )
		gr.setColor( borderColor )
		gr.circle( 'line', 400, 300, 20 )
		gr.circle( 'line', 400, 300, 40 )
		gr.rectangle( 'line', 400-110/2, 300-110/2, 110, 110 )
		gr.circle( 'line', 400, 300, 80 )
		gr.circle( 'line', 100, 100, 40 )
		gr.circle( 'line',  80, 100,  12 )
		gr.circle( 'line', 115, 100,  14 )
		gr.polygon( 'line',580,400,604,330,675,330,619,287,638,
					219,580,258,521,219,540,287,484,330,555,330)
		gr.polygon( 'line',150,10,150,110,200,160,250,110,250,60,400,60,
					400,110,350,160,300,110,300,210,450,210,450,10)
		gr.circle( 'line', 580, 298, 35 )
		gr.circle( 'line', 580, 298, 20 )
		gr.setCanvas()
	imagedata= canvas:newImageData()
	image	= gr.newImage( imagedata )
	end

function love.draw()
	gr.setColor( 1, 1, 1 )
	gr.draw( image, 0, 0 )
	drawMouse()
	addInstructions()
	end

function drawMouse()
	gr.setLineWidth(1)
	gr.setColor( colors[ fillColor ] )
	local mx, my = love.mouse.getPosition( )
	gr.circle( 'line', mx, my, 3 )
	gr.line( mx, my+5, mx, my + 10 )
	gr.line( mx, my-5, mx, my - 10 )
	gr.line( mx-5, my, mx - 10, my )
	gr.line( mx+5, my, mx + 10, my )
	end


function love.update( dt )
	if love.mouse.isDown( 1 ) then
		gr.setLineWidth(2)
		gr.clear()
		gr.setCanvas( canvas )
			gr.setColor( 1, 1, 1 )
			gr.draw( image )
			gr.setColor( borderColor )
			local mx, my = love.mouse.getPosition( )
			gr.circle( 'line', mx, my, 100 )
			gr.setColor( 1,1,1 )
			gr.setCanvas()
		imagedata = canvas:newImageData()
		image = gr.newImage( imagedata )
		end
	if love.mouse.isDown( 2 ) then
		mx, my = love.mouse.getPosition()
		image  = flood( mx, my, borderColor, colors[fillColor], imagedata )
		end
	end

function love.keypressed( key )
	if key == "escape" then love.event.quit() end
	if key == 'c' then fillColor = fillColor < #colors and fillColor + 1 or 1 print('color changed') end
	end

function addInstructions()
	gr.setColor( colors[ fillColor ] )
	gr.rectangle( 'line', 10, 510, 200, 60, 10, 10 )
	gr.print('"c" key to change fill color\nright click to fill area\nleft click to add circles to fill',20,520)
	end

function colorMatch( c1, c2 )
		return c1[1]==c2[1] and c1[2]==c2[2] and c1[3]==c2[3]
		end

function flood( x, y, bc, fc, imagedata )
-- starting x&y, border color, fill color, canvas imagedata
	local wd, ht = imagedata:getWidth(), imagedata:getHeight()

	local clr = { imagedata:getPixel( x, y ) }

	local stack = {
		{ x, y },
		push = function( self, x, y )			-- very large stack created!
			if x > 0 and x < wd and y > 0 and y < ht then
				self[ #self+1 ] = { x, y }	-- only check pixels on screen.
				end
			end,
		pop = function( self )
			local x, y = self[ #self ][1], self[ #self ][2]
			table.remove( self, #self )
			return x, y
			end
		}

	while #stack > 0 do					-- fill complete when stack empty
		local x, y	= stack:pop(  )
		local clr 	= { imagedata:getPixel( x, y ) }
		if not colorMatch( clr, bc ) and not colorMatch( clr, fc ) then
			imagedata:setPixel( x, y, unpack( fc ) )
			stack:push( x+1, y )		-- look at neighbors if color changed
			stack:push( x-1, y )		-- (very inefficient)
			stack:push( x, y+1 )
			stack:push( x, y-1 )
			end
		end

	return gr.newImage( imagedata )
	end
